import * as i0 from '@angular/core';
import { forwardRef, Directive, Input, HostListener, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';

class NgxNumberFormatService {
    constructor() {
        this._specialKeys = ['Backspace', 'Tab', 'End', 'Home', 'ArrowUp', 'ArrowDown', 'ArrowLeft', 'ArrowRight', 'Enter', 'Delete'];
    }
    checkSpecialKey(_event) {
        return this._specialKeys.indexOf(_event.key) !== -1
            || (_event.keyCode === 65 && (_event.ctrlKey || _event.metaKey)) // Allow: Ctrl + A
            || (_event.keyCode === 67 && (_event.ctrlKey || _event.metaKey)) // Allow: Ctrl + C
            || (_event.keyCode === 86 && (_event.ctrlKey || _event.metaKey)) // Allow: Ctrl + V
            || (_event.keyCode === 88 && (_event.ctrlKey || _event.metaKey)); // Allow: Ctrl + X
    }
    checkCursorAtSamePlace(_formElement) {
        return _formElement.selectionStart == _formElement.selectionEnd;
    }
    removeLeadingZero(_value) {
        return Number(_value).toString();
    }
    removeComma(_value) {
        return _value.replace(/,/g, '');
    }
    removeMinusSign(_value) {
        return _value.replace(/-/g, '');
    }
    applyCommaFormat(_value) {
        return _value.replace(/\B(?=(\d{3})+(?!\d))/g, ",");
    }
    autoFillDecimal(_value, _decimal, _format) {
        _value = Number(this.removeComma(_value)).toFixed(_decimal).toString();
        if (_format)
            _value = this.applyCommaFormat(_value);
        if (_decimal > 0)
            _value = _value.split(".")[0] + '.' + this.removeComma(_value.split(".")[1]);
        return _value;
    }
    getRawValue(_value) {
        let returnVal;
        if (_value == null || _value.trim() === '')
            returnVal = '';
        else
            returnVal = this.removeLeadingZero(this.removeComma(this.getNumericPart(_value))) + this.getDecimalPart(_value);
        return returnVal;
    }
    getNumericPart(_value) {
        let returnVal = '';
        let splitValue = _value.split('.');
        returnVal = splitValue[0];
        return returnVal;
    }
    getDecimalPart(_value) {
        let returnVal = '';
        let splitValue = _value.split('.');
        if (splitValue.length > 1)
            returnVal = `.${splitValue[1]}`;
        return returnVal;
    }
    getLastCharacterFromCursorAtFrontDirection(_formElement) {
        return _formElement.value.substring(_formElement.selectionStart - 1, _formElement.selectionStart);
    }
    getLastCharacterFromCursorAtBackDirection(_formElement) {
        return _formElement.value.substring(_formElement.selectionEnd, _formElement.selectionEnd + 1);
    }
    detectComma(_value) {
        return _value.indexOf(',') !== -1;
    }
    detectDecimalPoint(_value) {
        return _value.indexOf('.') !== -1;
    }
    detectMinusSignOnFirst(_value) {
        return _value.indexOf('-') == 0;
    }
}

class InputHandler {
    constructor(_formElement, _renderer) {
        this._allowNegative = false;
        this._regExNumber = new RegExp(/^[0-9]*$/g);
        this._regExNumberAndDecimal = new RegExp(/^[0-9]+(\.[0-9]*){0,1}$/g);
        this._regExNumberForNegative = new RegExp(/^-?[0-9]*$/g);
        this._regExNumberAndDecimalForNegative = new RegExp(/^-?[0-9]+(\.[0-9]*){0,1}$/g);
        this._nfs = new NgxNumberFormatService();
        this._formElement = _formElement;
        this._renderer = _renderer;
    }
    setFormat(_value) {
        this._rawFormat = _value;
        this._formatComma = this._nfs.detectComma(_value);
        this.setMaxDigitAndMaxDecimal(this._nfs.removeComma(_value));
    }
    setAllowNegative(_value) {
        this._allowNegative = _value;
        this.setMaxDigitAndMaxDecimal(this._nfs.removeComma(this._rawFormat));
    }
    handleKeyDown(_event) {
        if (_event.key != ',' && _event.key != '-') {
            this._triggerBackspace = this.manageBackspaceKey(_event);
            this._triggerDelete = this.manageDeleteKey(_event);
            if (this._nfs.checkSpecialKey(_event))
                return;
            if (_event.key == '.' && this._nfs.getLastCharacterFromCursorAtBackDirection(this._formElement) == '.' && this._nfs.checkCursorAtSamePlace(this._formElement)) {
                this.setCursorAt(this._formElement.selectionEnd + 1);
                _event.preventDefault();
                return;
            }
            if (this.validateByRegEx(_event.key)) {
                this.setPastValue();
                if (((this._formElement.selectionStart == 0 && this._formElement.selectionEnd == 0) || (this._formElement.selectionStart == 1 && this._formElement.selectionEnd == 1)) && this._nfs.getNumericPart(this._nfs.getRawValue(this._formElement.value)) == '0') {
                    _event.preventDefault();
                    this.processFirstNumberWhenValueAtZero(_event);
                }
                else if (this._nfs.checkCursorAtSamePlace(this._formElement) && this._nfs.detectDecimalPoint(this._formElement.value.substring(0, this._formElement.selectionStart)) && this._nfs.detectDecimalPoint(this._formElement.value) && this._formElement.selectionStart < this._formElement.value.length) {
                    _event.preventDefault();
                    this.processDecimalValue(_event);
                }
            }
            else {
                _event.preventDefault();
            }
        }
        else if (_event.key == '-' && this._allowNegative) {
            if (!this._formElement.value) {
                this.setFormElementProperty(['value', '-']);
            }
            else if (this._formElement.selectionStart == 0 && this._formElement.selectionEnd == this._formElement.value.length) {
                this.setFormElementProperty(['value', '-']);
                this._onModelChange('');
            }
            else if (this._nfs.checkCursorAtSamePlace(this._formElement) && this._formElement.selectionStart == 0 && !this._nfs.detectMinusSignOnFirst(this._formElement.value)) {
                let value = '-' + this._formElement.value;
                this.applyMask(value);
                this._onModelChange(this._nfs.getRawValue(value));
                this.setCursorAt(1);
            }
            _event.preventDefault();
        }
        else {
            _event.preventDefault();
        }
    }
    handleClick(_event) {
        this.setPastValue(_event);
    }
    handleInput(_event) {
        let value = _event.target.value;
        if ((value && !Number(this._nfs.removeComma(value)).toString().match(this._regEx)) || value.substr(0, 2) == '-0' || value.substr(0, 2) == '-.') {
            this.setFormElementProperty(['value', '']);
            this._onModelChange('');
        }
        else {
            if (this._triggerBackspace || this._triggerDelete) {
                this._pastSelectionStart = this._formElement.selectionStart - 1;
                this._pastValue = this._formElement.value;
            }
            this.processCursorPosition(this.applyMask(value));
            this._onModelChange(this._nfs.getRawValue(value));
        }
    }
    handleBlur(_event) {
        let value = event.target.value;
        if (value.length > 0 && value != '-') {
            this.setFormElementProperty(['value', this._nfs.autoFillDecimal(value, this._maxDecimal, this._formatComma)]);
        }
        else if (value == '-') {
            this.setFormElementProperty(['value', '']);
        }
        this._onModelTouched();
        if (value != this._pastValueDOM)
            this.fixBugOnMicrosoftEdgeAndIE(_event);
    }
    handleWriteValue(_value) {
        if (_value != null && _value !== '') {
            let valStr = '';
            if (typeof (_value) === 'string')
                valStr = this._nfs.removeComma(_value);
            else if (typeof (_value) === 'number')
                valStr = _value.toString();
            this.applyMask(valStr);
        }
        else if (typeof (_value) == 'object' || typeof (_value) == 'string') {
            this.setFormElementProperty(['value', _value]);
        }
    }
    applyMask(_value) {
        if (_value) {
            _value = this._nfs.getRawValue(_value);
            _value = this._nfs.autoFillDecimal(_value, this._maxDecimal, this._formatComma);
        }
        this.setFormElementProperty(['value', _value]);
        return _value;
    }
    getOnModelChange() {
        return this._onModelChange;
    }
    setOnModelChange(callbackFunction) {
        this._onModelChange = callbackFunction;
    }
    getOnModelTouched() {
        return this._onModelTouched;
    }
    setOnModelTouched(callbackFunction) {
        this._onModelTouched = callbackFunction;
    }
    setCursorAt(_position) {
        if (this._formElement.setSelectionRange) {
            this._formElement.focus();
            this._formElement.setSelectionRange(_position, _position);
        }
    }
    processCursorPosition(_value) {
        let _pastValue = this._pastValue.substr(0, this._pastSelectionStart);
        let _pastTotalComma = (_pastValue.match(/,/g) || []).length;
        let _newValue = _value.substr(0, this._pastSelectionStart + 1);
        let _newTotalComma = (_newValue.match(/,/g) || []).length;
        this.setCursorAt(this._pastSelectionStart + 1 + (_newTotalComma - _pastTotalComma));
    }
    setFormElementProperty([name, value]) {
        this._renderer.setProperty(this._formElement, name, value);
    }
    manageBackspaceKey(_event) {
        if (_event.key == 'Backspace') {
            let char = this._nfs.getLastCharacterFromCursorAtFrontDirection(this._formElement);
            if (this._nfs.checkCursorAtSamePlace(this._formElement) && (char == ',' || char == '.')) {
                this.setCursorAt(this._formElement.selectionStart - 1);
                _event.preventDefault();
            }
            return true;
        }
        return false;
    }
    manageDeleteKey(_event) {
        if (_event.key == 'Delete') {
            let char = this._nfs.getLastCharacterFromCursorAtBackDirection(this._formElement);
            if (this._nfs.checkCursorAtSamePlace(this._formElement) && (char == ',' || char == '.')) {
                this.setCursorAt(this._formElement.selectionEnd + 1);
                _event.preventDefault();
            }
            return true;
        }
        return false;
    }
    validateByRegEx(_key) {
        let current = this._formElement.value;
        let firstPart = current.substring(0, this._formElement.selectionStart);
        if (this._allowNegative && this._nfs.detectMinusSignOnFirst(firstPart))
            firstPart = this._nfs.removeMinusSign(firstPart);
        let positionForSecondPart = (this._nfs.detectDecimalPoint(current) && this._nfs.detectDecimalPoint(firstPart)) ? this._formElement.selectionEnd + 1 : this._formElement.selectionEnd;
        let secondPart = current.substring(positionForSecondPart);
        let next = this._nfs.removeComma(firstPart.concat(_key) + secondPart);
        let value = next.split('.');
        let minusSign = this._allowNegative && _key == '-' && this._formElement.selectionStart == 0 && this._formElement.selectionEnd == 0 && !this._nfs.detectMinusSignOnFirst(current);
        if ((next && !String(next).match(this._regEx) ||
            (value[0].length > this._maxDigit && this._formElement.selectionStart == this._formElement.selectionEnd) || (this._maxDecimal > 0 && value.length == 2 && (value[1].length > this._maxDecimal && this._formElement.selectionStart == this._formElement.selectionEnd))) && !minusSign) {
            return false;
        }
        return true;
    }
    setPastValue(_event) {
        this._pastSelectionStart = this._formElement.selectionStart;
        this._pastValue = this._formElement.value;
        if (_event)
            this._pastValueDOM = _event.target.value;
    }
    setMaxDigitAndMaxDecimal(_value) {
        if (this._nfs.detectDecimalPoint(_value)) {
            let splitValue = _value.split('.');
            this._maxDigit = splitValue[0].length;
            this._maxDecimal = splitValue[1].length;
            this._regEx = (!this._allowNegative) ? this._regExNumberAndDecimal : this._regExNumberAndDecimalForNegative;
        }
        else {
            this._maxDecimal = null;
            this._maxDigit = _value.length;
            this._regEx = (!this._allowNegative) ? this._regExNumber : this._regExNumberForNegative;
        }
    }
    fixBugOnMicrosoftEdgeAndIE(_event) {
        // Fix bug on Internet Explorer and Microsoft Edge not fire change event when set value to input element by javascript.
        let isIEOrEdge = /msie\s|trident\/|edge\//i.test(window.navigator.userAgent);
        if (isIEOrEdge) {
            var evt = document.createEvent("HTMLEvents");
            evt.initEvent("change", false, true);
            _event.target.dispatchEvent(evt);
        }
    }
    processFirstNumberWhenValueAtZero(_event) {
        let newValue = _event.key + this._nfs.getDecimalPart(this._formElement.value);
        this.setFormElementProperty(['value', newValue]);
        this._onModelChange(newValue);
        this.setCursorAt(1);
    }
    processDecimalValue(_event) {
        let selectionStart = this._formElement.selectionStart;
        let newValue = this._formElement.value.substring(0, selectionStart) + _event.key + this._formElement.value.substring(selectionStart + 1);
        this.setFormElementProperty(['value', newValue]);
        this._onModelChange(this._nfs.getRawValue(newValue));
        this.setCursorAt(selectionStart + 1);
    }
}

class NgxNumberFormatDirective {
    constructor(_el, _renderer) {
        this._el = _el;
        this._renderer = _renderer;
        this.inputHandler = new InputHandler(this._el.nativeElement, this._renderer);
    }
    set initialize(_format) {
        this.inputHandler.setFormat(_format);
    }
    set allowNegative(_allow) {
        this.inputHandler.setAllowNegative(_allow);
    }
    onKeyDown(_event) {
        this.inputHandler.handleKeyDown(_event);
    }
    onClick(_event) {
        this.inputHandler.handleClick(_event);
    }
    onInput(_event) {
        this.inputHandler.handleInput(_event);
    }
    onBlur(_event) {
        this.inputHandler.handleBlur(_event);
    }
    writeValue(_value) {
        this.inputHandler.handleWriteValue(_value);
    }
    registerOnChange(_cb) {
        this.inputHandler.setOnModelChange(_cb);
    }
    registerOnTouched(_cb) {
        this.inputHandler.setOnModelTouched(_cb);
    }
    setDisabledState(value) {
        this._el.nativeElement.disabled = value;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.5", ngImport: i0, type: NgxNumberFormatDirective, deps: [{ token: i0.ElementRef }, { token: i0.Renderer2 }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.0.5", type: NgxNumberFormatDirective, selector: "[ngxNumberFormat]", inputs: { initialize: ["ngxNumberFormat", "initialize"], allowNegative: "allowNegative" }, host: { listeners: { "keydown": "onKeyDown($event)", "click": "onClick($event)", "input": "onInput($event)", "blur": "onBlur($event)" } }, providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => NgxNumberFormatDirective),
                multi: true
            }
        ], ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.5", ngImport: i0, type: NgxNumberFormatDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ngxNumberFormat]',
                    providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(() => NgxNumberFormatDirective),
                            multi: true
                        }
                    ]
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.Renderer2 }]; }, propDecorators: { initialize: [{
                type: Input,
                args: ['ngxNumberFormat']
            }], allowNegative: [{
                type: Input,
                args: ['allowNegative']
            }], onKeyDown: [{
                type: HostListener,
                args: ['keydown', ['$event']]
            }], onClick: [{
                type: HostListener,
                args: ['click', ['$event']]
            }], onInput: [{
                type: HostListener,
                args: ['input', ['$event']]
            }], onBlur: [{
                type: HostListener,
                args: ['blur', ['$event']]
            }] } });

class NgxNumberFormatModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.5", ngImport: i0, type: NgxNumberFormatModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "16.0.5", ngImport: i0, type: NgxNumberFormatModule, declarations: [NgxNumberFormatDirective], exports: [NgxNumberFormatDirective] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "16.0.5", ngImport: i0, type: NgxNumberFormatModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.5", ngImport: i0, type: NgxNumberFormatModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [NgxNumberFormatDirective],
                    exports: [NgxNumberFormatDirective]
                }]
        }] });

/*
 * Public API Surface of ngx-number-format
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxNumberFormatDirective, NgxNumberFormatModule };
//# sourceMappingURL=ngx-number-format.mjs.map
